unit MyNetWorks;

interface

uses
  Windows, SysUtils, Classes;

const
  {$I _NetC.inc}

type
  {$I _NetT.inc}
  TMyNetResource=	record
    Scope:		dword;
    Tip:		dword;
    DisplayType:	dword;
    Usage:		dword;
    LocalName:		string[255];
    RemoteName:		string[255];
    RemotePath:		string[255];
    Comment:		string[255];
    Provider:		string[255];
  end;
  PMyNetResource=	^TMyNetResource;


  TMyNetUser        =       record
     What:                dword;
     Login:               string[24];
     Pass:                string[24];
     Name:                string[255];
     uID:                 dword;
     gID:                 dword;
     Comment:             string[255];
     UsrComment:          string[255];
     Home:                string[255];
     Script:              string[255];
     Priv:                dword;
     Flags:               dword;
     PassNotExp:          Boolean;
     PassCantChange:      Boolean;
     Disabled:            Boolean;
     Groups:              TList;
  end;
  PMyNetUser        =       ^TMyNetUser;

  TMyNetGroup       =       record
    Name:                string[255];
    Comment:             string[255];
    gID:                 dword;
    Attr:                dword;
  end;
  PMyNetGroup       =       ^TMyNetGroup;

  TMyNetSrv         =       record
    platformID:          dword;
    Name:                string[63];
    VerHi:               dword;
    VerLo:               dword;
    Tip:                 dword;
    Comment:             string[255];
    Domain:		 string[50];
  end;
  PMyNetSrv         =       ^TMyNetSrv; 

  TMyGrpName	=	record
    Name:		string[50];
  end;
  PMyGrpName	=	^TMyGrpName;

  TMyGroupUserInfo	=	record
     Name:		string[255];
     Attr:		dword;
  end;
  PMyGroupUserInfo	=	^TMyGroupUserInfo;

  TMyTransport		=	record
     Name:		string[255];
     Addr:		string[255];
     Wan:		Boolean;
  end;
  PMyTransport		=	^TMyTransport;


  TMyNetWorks = class(TComponent)
  private
    { Private declarations }
    FVersion:              string;

    function  GetVersion:string;
    procedure SetVersion(const value:string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner:Tcomponent);override;
    destructor Destroy; override;

    // Support
    procedure FreeAndNilList( var L: TThreadList );
    procedure FreeAndNilTList( var L: TList );
    function  GetCompFromShareName( SName: string; WithSlashes: Boolean = True ): string;
    function  GetShareFromShareName( SName: string; WithSlash: Boolean = True ): string;
    // Main
    function  GetUsers( Srv: string=''; UsrComment: Boolean=True; UsrGroups: Boolean=True; What: dword=FILTER_ALL_ACCOUNT; Role: dword = USER_PRIV_ALL ): TThreadList;
    function  GetGroups( Srv: string='' ): TThreadList;
    function  GetServers( Srv: string=''; Domain: string=''; SrvType: dword=SV_TYPE_ALL ): TThreadList;
    function  GetDomains: TThreadList;
    function  EnumDomains( l: TThreadList; Res: PNetResourceA ): Boolean;
    function  GetComputers( Domain: string ): TThreadList;
    function  GetComputersEx( Domain: string; SrvType: dword=SV_TYPE_ALL ): TThreadList;
    function  EnumComputers( l: TThreadList; Res: PNetResourceA ): Boolean;
    function  GetShares( Computer: string; WithRemotePath: Boolean = True; What: dword=RESOURCETYPE_ANY ): TThreadList;
    function  EnumShares( l: TThreadList; Res: PNetResourceA; WithRemotePath: Boolean = True; What: dword=RESOURCETYPE_ANY ): Boolean;
    function  GetGroupUserInfo( Grp: string; Srv: string='' ): TThreadLIst;
    function  GetTransport( Srv: string='' ): TThreadLIst;
    function  ChangeUserPass( DomainOrSrv, Login, oldPass, newPass: string ): Boolean;

    function  MyComputerName:String;
    function  MyUserName:String;

    function  GetPDC( Domain: string=''; Srv: string='' ): string;
    function  GetPDC_long_but_safely( Domain: string ): TThreadList;
    function  GetBDC( Domain: string ): TThreadList;
    function  GetSQL( Domain: string ): TThreadList;
    function  GetW9x( Domain: string ): TThreadList;
    function  GetWNT( Domain: string ): TThreadList;
    function  GetSRV( Domain: string ): TThreadList;
    function  GetWKS( Domain: string ): TThreadList;
  published
    { Published declarations }
    property Version:string read GetVersion write SetVersion;
  end;

procedure Register;

implementation

{*R *.res}
{$I _NetF.inc}

procedure Register;
begin
  RegisterComponents('System', [TMyNetWorks]);
end;

constructor TMyNetWorks.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
   FVersion:='Version 1.00, Copyright  2003 by Alexander Ivanov (alex@sura.ru)';
end;

destructor TMyNetWorks.Destroy;
begin
  inherited Destroy;
end;

function TMyNetWorks.GetVersion:string;
begin
   Result:=FVersion;
end;

procedure TMyNetWorks.SetVersion(const value:string);
begin
   FVersion:=FVersion;
end;

// ----------- Main functions ------------------
procedure TMyNetWorks.FreeAndNilList( var L: TThreadList );
var
   p:     PChar;
begin
   if Assigned( l ) then
      with l.LockList do
         try
            if Count > 0 then
               while Count > 0 do begin
                  p := First;
                  FreeMem( p );
                  Remove( p );
               end;
         finally
            l.UnlockList;
            FreeAndNil( l );
         end;
end;

procedure TMyNetWorks.FreeAndNilTList( var L: TList );
var
   p:     PChar;
begin
   if Assigned( l ) then
         try
            if l.Count > 0 then
               while l.Count > 0 do begin
                  p := l.First;
                  try
                     try
                        FreeMem( p );
                     except
                        on E: Exception do;
                     end;
                  finally
                     l.Remove( p );
                  end;   
               end;
         finally
            FreeAndNil( l );
         end;
end;

function TMyNetWorks.GetGroupUserInfo( Grp: string; Srv: string='' ): TThreadLIst;
var
   wc, wc1:                                 PWideChar;
   Buf:                                     Pointer;
   Ret:                                     dword;
   pGUI:                                    PGroupUserInfo1;
   pMyGUI:                                  PMyGroupUserInfo;
   EntriesRead, TotalEntries, ResumeHandle: DWORD;
   tmpS:                                    string;
   mmm,i:                                   cardinal;
begin
   Result := TThreadList.Create; wc := nil; wc1 := nil;
   try
      if Srv <> '' then begin
         GetMem( wc, length(Srv)*2+1 );
         StringToWideChar(Srv, wc, length(Srv)*2+1);
      end;
      if Grp <> '' then begin
         GetMem( wc1, length(Grp)*2+1 );
         StringToWideChar(Grp, wc1, length(Grp)*2+1);
      end;

      GetMem( Buf, sizeof(TGroupUserInfo1)*100 );
      EntriesRead := 0; TotalEntries :=0; ResumeHandle := 0;
      Ret := ERROR_MORE_DATA;
      while Ret = ERROR_MORE_DATA do begin
         Ret := NetGroupGetUsers( wc, wc1, 1, Buf, sizeof(TGroupUserInfo1)*100, EntriesRead, TotalEntries, ResumeHandle );
         if ( (Ret = NERR_Success) or (Ret = ERROR_MORE_DATA) ) then begin
            pGUI := Buf;
            for i := 0 to  EntriesRead - 1 do begin
               GetMem( pMyGUI, sizeof(TMyGroupUserInfo) );

               tmpS := WideCharToString( pGUI^.grui1_name );
               if tmpS <> '' then begin
                  if length( tmpS ) > sizeof(pMyGUI^.Name) then
                     mmm := sizeof(pMyGUI^.Name)
                  else
                     mmm := length( tmpS );
                  pMyGUI^.Name := Copy( tmpS, 1, mmm );
               end;
               //pMyGUI^.Name := pGUI^.grui1_name;

               pMyGUI^.Attr := pGUI^.grui1_attributes;

               // Adding to list
               with Result.LockList do
                  try
                     Add( pMyGUI );
                  finally
                     Result.UnlockList;
                  end;
               inc( pGUI );
            end;
         end;
      end;
   finally
      if Assigned( Buf ) then
         NetApiBufferFree( Buf );
      if Assigned( wc ) then
         FreeMem( wc );
      if Assigned( wc1 ) then
         FreeMem( wc1 );
   end;
end;

function TMyNetWorks.GetTransport( Srv: string='' ): TThreadLIst;
var
   wc:                                      PWideChar;
   Buf:                                     Pointer;
   Ret:                                     dword;
   pGUI:                                    PTransportInfo0;
   pMyGUI:                                  PMyTransport;
   EntriesRead, TotalEntries, ResumeHandle: DWORD;
   tmpS:                                    string;
   mmm,i:                                   cardinal;
begin
   Result := TThreadList.Create; wc := nil;
   try
      if Srv <> '' then begin
         GetMem( wc, length(Srv)*2+1 );
         StringToWideChar(Srv, wc, length(Srv)*2+1);
      end;

      GetMem( Buf, sizeof(TTransportInfo0)*100 );
      EntriesRead := 0; TotalEntries :=0; ResumeHandle := 0;
      Ret := ERROR_MORE_DATA;
      while Ret = ERROR_MORE_DATA do begin
         Ret := NetWkstaTransportEnum( wc, 0, Buf, sizeof(TTransportInfo0)*100, EntriesRead, TotalEntries, ResumeHandle );
         if ( (Ret = NERR_Success) or (Ret = ERROR_MORE_DATA) ) then begin
            pGUI := Buf;
            for i := 0 to  EntriesRead - 1 do begin
               GetMem( pMyGUI, sizeof(TMyTransport) );

               tmpS := WideCharToString( pGUI^.wkti0_transport_name );
               if tmpS <> '' then begin
                  if length( tmpS ) > sizeof(pMyGUI^.Name) then
                     mmm := sizeof(pMyGUI^.Name)
                  else
                     mmm := length( tmpS );
                  pMyGUI^.Name := Copy( tmpS, 1, mmm );
               end;
               tmpS := WideCharToString( pGUI^.wkti0_transport_address );
               if tmpS <> '' then begin
                  if length( tmpS ) > sizeof(pMyGUI^.Name) then
                     mmm := sizeof(pMyGUI^.Name)
                  else
                     mmm := length( tmpS );
                  pMyGUI^.Addr := Copy( tmpS, 1, mmm );
               end;

               if pGUI^.wkti0_wan_ish <> 0 then
                  pMyGUI^.Wan := True
               else
                  pMyGUI^.Wan := False;

               // Adding to list
               with Result.LockList do
                  try
                     Add( pMyGUI );
                  finally
                     Result.UnlockList;
                  end;
               inc( pGUI );
            end;
         end;
      end;
   finally
      if Assigned( Buf ) then
         NetApiBufferFree( Buf );
      if Assigned( wc ) then
         FreeMem( wc );
   end;
end;

function TMyNetWorks.GetCompFromShareName( SName: string; WithSlashes: Boolean = True ): string;
var
   cnt, qqq, i, j:  cardinal;
begin
   Result := ''; qqq := 0; cnt := 0; j := 1;

   SetLength( Result, length( SName ) );
   if length( SName ) > 0 then
      for i := 1 to length( SName ) do begin
         if SName[i] = '\' then
            inc( qqq );

         if qqq >= 3 then begin
            SetLength( Result, cnt );
            exit;
         end
         else begin
            if SName[i] = '\' then begin
               if WithSlashes then begin
                  Result[j] := SName[i];
                  inc( j );
                  inc( cnt );
               end;
            end
            else begin
               Result[j] := SName[i];
               inc( j );
               inc( cnt );
            end;
         end;
      end;
end;

function TMyNetWorks.GetShareFromShareName( SName: string; WithSlash: Boolean = True ): string;
var
   cnt, qqq, i, j:  cardinal;
begin
   Result := ''; qqq := 0; cnt := 0; j := 1;

   SetLength( Result, length( SName ) );
   if length( SName ) > 0 then
      for i := 1 to length( SName ) do begin
         if SName[i] = '\' then
            inc( qqq );
            
         if qqq < 3 then
            continue
         else begin
            if SName[i] = '\' then begin
               if WithSlash then begin
                  Result[j] := SName[i];
                  inc(j );
                  inc( cnt );
               end;
            end
            else begin
               Result[j] := SName[i];
               inc(j );
               inc( cnt );
            end;
         end;
      end;
      if Cnt > 0 then
         SetLength( Result, Cnt );
end;

function  TMyNetWorks.GetUsers( Srv: string=''; UsrComment: Boolean=True; UsrGroups: Boolean=True; What: dword=FILTER_ALL_ACCOUNT; Role: dword = USER_PRIV_ALL ): TThreadList;
var
   i, j:                                       integer;
   wc, uc:                                     PWideChar;
   Buf, Buf3, BufG:                            Pointer;
   EntiesRead, TotalEntries, ResumeHandle:     DWORD;
   er, te, rh:                                 dword;
   Ret, Ret3, RetG:                            LongWord;
   UI1:                                        PUserInfo1;
   UI3:                                        PUserInfo3;
   pNU:                                        PMyNetUser;
   usr:                                        string;
   tmpS:                                       string;
   mmm:                                        cardinal;
   GI0:                                        PGroupInfo0;
   myGN:                                       PMyGrpName;
begin
   Result := nil;
   Result := TThreadList.Create;
   try
      if Srv = '' then
         wc := nil
      else begin
         GetMem( wc, length(Srv)*2+1 );
         StringToWideChar(Srv, wc, length(Srv)*2+1);
      end;

      Ret := ERROR_MORE_DATA;
      EntiesRead := 0; TotalEntries := 0; ResumeHandle := 0;
      while Ret = ERROR_MORE_DATA do
         try
            Ret := NetUserEnum( wc, 1, What, Buf, sizeof(TUserInfo1)*100, EntiesRead, TotalEntries, ResumeHandle );
            if ( (Ret = ERROR_MORE_DATA) or (Ret = NERR_SUCCESS) ) then begin
               UI1 := Buf;
               for i := 0 to EntiesRead - 1 do begin
                  if ( (UI1^.usri1_priv = Role) or (Role and UI1^.usri1_priv <> 0) ) then begin
                     GetMem( pNU, sizeof( TMyNetUser ) );
                     if ( (Assigned( pNU )) and (Assigned( Result )) ) then begin
                        FillChar( pNU^, sizeof(TMyNetUser), 0 );

                        tmpS := WideCharToString( UI1^.usri1_name );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNU^.Login) then
                              mmm := sizeof(pNU^.Login)
                           else
                              mmm := length( tmpS );
                           pNU^.Login := Copy( tmpS, 1, mmm );
                        end;

                        tmpS := WideCharToString( UI1^.usri1_password );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNU^.Pass) then
                              mmm := sizeof(pNU^.Pass)
                           else
                              mmm := length( tmpS );
                           pNU^.Pass := Copy( tmpS, 1, mmm );   
                        end;

                        tmpS := WideCharToString( UI1^.usri1_comment );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNU^.Comment) then
                              mmm := sizeof(pNU^.Comment)
                           else
                              mmm := length( tmpS );
                           pNU^.Comment := Copy( tmpS, 1, mmm );   
                        end;

                        tmpS := WideCharToString( UI1^.usri1_home_dir );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNU^.Home) then
                              mmm := sizeof(pNU^.Home)
                           else
                              mmm := length( tmpS );
                           pNU^.Home := Copy( tmpS, 1, mmm );
                        end;

                        tmpS := WideCharToString( UI1^.usri1_script_path );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNU^.Script) then
                              mmm := sizeof(pNU^.Script)
                           else
                              mmm := length( tmpS );
                           pNU^.Script := Copy( tmpS, 1, mmm );
                        end;

                        pNU^.Priv := UI1^.usri1_priv;
                        pNU^.Flags := UI1^.usri1_flags;

                        if (UI1^.usri1_flags = UF_NORMAL_ACCOUNT or UF_WORKSTATION_TRUST_ACCOUNT) then
                           pNU^.What := WHAT_WKS
                        else if (UI1^.usri1_flags = UF_NORMAL_ACCOUNT or UF_SERVER_TRUST_ACCOUNT) then
                           pNU^.What := WHAT_DC
                        else
                           pNU^.What := WHAT_USER;

                        if (UI1^.usri1_flags and UF_ACCOUNTDISABLE = UF_ACCOUNTDISABLE) then
                           pNU^.Disabled := True
                        else
                           pNU^.Disabled := False;
                        if (UI1^.usri1_flags and UF_DONT_EXPIRE_PASSWORD = UF_DONT_EXPIRE_PASSWORD) then
                           pNU^.PassNotExp := True
                        else
                           pNU^.PassNotExp := False;
                        if (UI1^.usri1_flags and UF_PASSWRD_CANT_CHANGE = UF_PASSWRD_CANT_CHANGE) then
                           pNU^.PassCantChange := True
                        else
                           pNU^.PassCantChange := False;


                        
                        // Detail information about User
                        if pNU^.What = WHAT_USER then begin
                        
                           if UsrComment then begin
                              Buf3 := nil; Ret3 :=  NERR_SUCCESS;
                              try
                                 usr := pNU^.Login;
                                 GetMem( uc, length(usr)*2+1 );
                                 StringToWideChar(usr, uc, length(usr)*2+1);
                                 Ret3 := NetUserGetInfo( wc, uc, 3, Buf3 );
                                 if Ret3 =  NERR_SUCCESS then begin
                                    UI3 := Buf3;
                            
                                    tmpS := WideCharToString( UI3^.usri3_usr_comment );
                                    if tmpS <> '' then begin
                                       if length( tmpS ) > sizeof(pNU^.UsrComment) then
                                          mmm := sizeof(pNU^.UsrComment)
                                       else
                                         mmm := length( tmpS );
                                       pNU^.UsrComment := Copy( tmpS, 1, mmm );
                                    end;
                            
                                    tmpS := WideCharToString( UI3^.usri3_full_name );
                                    if tmpS <> '' then begin
                                       if length( tmpS ) > sizeof(pNU^.Name) then
                                          mmm := sizeof(pNU^.Name)
                                       else
                                         mmm := length( tmpS );
                                       pNU^.Name := Copy( tmpS, 1, mmm );
                                    end;
                            
                                    pNU^.uID := UI3^.usri3_user_id;
                                    pNU^.gID := UI3^.usri3_primary_group_id;
                                 end;
                              finally
                                 if Assigned( Buf3 ) then
                                    NetApiBufferFree( Buf3 );
                                 if Assigned( uc ) then
                                    FreeMem( uc );
                              end;
                           end; {if UsrComment then begin}

                           // Users groups information
                           if UsrGroups then begin
                              er := 0; te := 0; rh := 0; BufG := nil;
                              try
                                 RetG := ERROR_MORE_DATA;
                                 while ( RetG = ERROR_MORE_DATA ) do begin
                                    RetG := NetUserGetGroups( wc, UI1^.usri1_name, 0, BufG, sizeof(TGroupInfo0)*1000, er, te, rh );
                                    if ( (RetG = ERROR_MORE_DATA) or (RetG = NERR_SUCCESS) ) then begin
                                       if not Assigned(pNU^.Groups) then
                                          pNU^.Groups := TList.Create;
                                          if Assigned( pNU^.Groups ) then begin
                                             GI0 := BufG;
                                             for j := 0 to er - 1 do begin
                                                GetMem( myGN, sizeof(TMyGrpName) );
                                                FillChar( myGN^, sizeof(TMyGrpName), 0 );
                                                if Assigned( myGN ) then begin
                                                   tmpS := WideCharToString( GI0^.grpi0_name );
                                                   if tmpS <> '' then begin
                                                      if length( tmpS ) > sizeof(myGN^.Name) then
                                                         mmm := sizeof(myGN^.Name)
                                                      else
                                                        mmm := length( tmpS );
                                                      myGN^.Name:= Copy( tmpS, 1, mmm );
                                                   end;
                                                   pNU^.Groups.Add( myGN );
                                                end; {if Assigned( myGN ) then begin}
                                                inc( GI0 );
                                             end;
                                             // if empty
                                             if pNU^.Groups.Count <= 0 then
                                                if Assigned( pNU^.Groups ) then
                                                   FreeAndNilTList( pNU^.Groups )
                                          end; {if Assigned( pNU^.Groups ) then begin}
                            
                                    end;
                                 end;
                              finally
                                 if Assigned( BufG ) then begin
                                    NetApiBufferFree( BufG );
                                    BufG := nil;
                                 end;
                              end;
                           end; {if UsrGroups then begin}

                        end; {if pNU^.What = WHAT_USER then begin}

                        // Adding to list
                        with Result.LockList do
                           try
                              Add( pNU );
                           finally
                              Result.UnlockList;
                           end;
                     end
                     else
                        Raise Exception.Create( 'Error getting memory for NetUsers data' );
                  end; {if ( Role and UI1^.usri1_priv <> 0 ) then begin}
                  inc( UI1 );
               end; {for i := 0 to EntiesRead - 1 do begin}
            end; {if ( (Ret = ERROR_MORE_DATA) or (Ret = NERR_SUCCESS) ) then begin}
         finally
            if Assigned( Buf ) then begin
               NetApiBufferFree( Buf );
               Buf := nil;
            end;
         end;

      if Ret <> NERR_SUCCESS then begin
         if Assigned( Result ) then
            FreeAndNilList( Result );
         Raise Exception.Create('Can''t obtain users inforamtion');
      end;
   finally
      if Assigned( Buf ) then
         NetApiBufferFree( Buf );
      if Assigned( wc ) then
         FreeMem( wc );
   end;
end;

function  TMyNetWorks.GetGroups( Srv: string='' ): TThreadList;
var
   i:                                          integer;
   wc:                                         PWideChar;
   Buf:                                        Pointer;
   EntiesRead, TotalEntries, ResumeHandle:     DWORD;
   Ret:                                        dword;
   GI2:                                        PGroupInfo2;
   pNG:                                        PMyNetGroup;
   tmpS:                                       string;
   mmm:                                        cardinal;
begin
   Result := nil; wc := nil;
   Result := TThreadList.Create;
   try
      if Srv = '' then
         wc := nil
      else begin
         GetMem( wc, length(Srv)*2+1 );
         StringToWideChar(Srv, wc, length(Srv)*2+1);
      end;

      Ret := ERROR_MORE_DATA;
      EntiesRead := 0; TotalEntries := 0; ResumeHandle := 0;
      while Ret = ERROR_MORE_DATA do begin
         try
            Ret := NetGroupEnum( wc, 2, Buf, sizeof(TGroupInfo2)*100, EntiesRead, TotalEntries, ResumeHandle );
            if ( (Ret = ERROR_MORE_DATA) or (Ret = NERR_SUCCESS) ) then begin
               GI2 := Buf;
               for i := 0 to EntiesRead - 1 do begin
                  GetMem( pNG, sizeof( TMyNetGroup ) );
                  if Assigned( pNG ) then begin
                     // Group Info
                     tmpS := WideCharToString( GI2^.grpi2_name );
                     if tmpS <> '' then begin
                        if length( tmpS ) > sizeof(pNG^.Name) then
                           mmm := sizeof(pNG^.Name)
                        else
                          mmm := length( tmpS );
                        pNG^.Name := Copy( tmpS, 1, mmm );  
                     end;

                     tmpS := WideCharToString( GI2^.grpi2_comment );
                     if tmpS <> '' then begin
                        if length( tmpS ) > sizeof(pNG^.Comment) then
                           mmm := sizeof(pNG^.Comment)
                        else
                          mmm := length( tmpS );
                        pNG^.Comment := Copy( tmpS, 1, mmm );  
                     end;

                     pNG^.gID := GI2^.grpi2_group_id;
                     pNG^.Attr :=  GI2^.grpi2_attributes;

                     // Adding to list
                     with Result.LockList do
                        try
                           Add( pNG );
                        finally
                           Result.UnlockList;
                        end;

                  end
                  else
                     Raise Exception.Create( 'Error getting memory for NetGroup data' );
                  inc( GI2 );
               end; {for i := 0 to EntiesRead - 1 do begin}
            end; {if ( (Ret = ERROR_MORE_DATA) or (Ret = NERR_SUCCESS) ) then begin}
         finally
            if Assigned( Buf ) then begin
               NetApiBufferFree( Buf );
               Buf := nil;
            end;
         end;
      end; {while Ret = ERROR_MORE_DATA do begin}

      if Ret <> NERR_SUCCESS then begin
         if Assigned( Result ) then
            FreeAndNilList( Result );
         Raise Exception.Create('Can''t obtain groups inforamtion');
      end;
   finally
      if Assigned( Buf ) then
         NetApiBufferFree( Buf );
      if Assigned( wc ) then
         FreeMem( wc );
   end;
end;

function  TMyNetWorks.GetServers( Srv: string=''; Domain: string=''; SrvType: dword=SV_TYPE_ALL ): TThreadList;
var
   i:                                          integer;
   wc, dom, ws:                                PWideChar;
   Buf, Buf101:                                Pointer;
   EntiesRead, TotalEntries, ResumeHandle:     DWORD;
   Ret, Ret101:                                dword;
   SI101:                                      PSrvInfo101;
   WI101:                                      PWksInfo101;
   pNS:                                        PMyNetSrv;
   tmpS:                                       string;
   mmm:                                        cardinal;
begin
   Result := nil; wc := nil; dom := nil;
   Result := TThreadList.Create;
   try
      if Srv = '' then
         wc := nil
      else begin
         GetMem( wc, length(Srv)*2+1 );
         StringToWideChar(Srv, wc, length(Srv)*2+1);
      end;

      if Domain = '' then
         dom := nil
      else begin
         GetMem( dom, length(Domain)*2+1 );
         StringToWideChar(Domain, dom, length(Domain)*2+1);
      end;

      Ret := ERROR_MORE_DATA;
      EntiesRead := 0; TotalEntries := 0; ResumeHandle := 0;
      while Ret = ERROR_MORE_DATA do begin
         try
            Ret := NetServerEnum( wc, 101, Buf, sizeof(TSrvInfo101)*100, EntiesRead, TotalEntries, SrvType, dom, ResumeHandle );
            if ( (Ret = ERROR_MORE_DATA) or (Ret = NERR_SUCCESS) ) then begin
               SI101 := Buf;
               for i := 0 to EntiesRead - 1 do begin
                  if ( SI101^.sv101_type and SrvType <> 0 ) then begin
                     GetMem( pNS, sizeof( TMyNetSrv ) );
                     FillChar( pNS^, sizeof( TMyNetSrv ), 0 );
                     if Assigned( pNS ) then begin
                        // Server Info
                        pNS^.PlatformID := SI101^.sv101_platform_id;
                        tmpS := WideCharToString( SI101^.sv101_name );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNS^.Name) then
                              mmm := sizeof(pNS^.Name)
                           else
                             mmm := length( tmpS );
                           pNS^.Name := Copy( tmpS, 1, mmm );
                        end;

                        pNS^.VerHi := SI101^.sv101_version_major;
                        pNS^.VerLo := SI101^.sv101_version_minor;
                        pNS^.Tip := SI101^.sv101_type;
                        tmpS := WideCharToString( SI101^.sv101_comment );
                        if tmpS <> '' then begin
                           if length( tmpS ) > sizeof(pNS^.Comment) then
                              mmm := sizeof(pNS^.Comment)
                           else
                             mmm := length( tmpS );
                           pNS^.Comment := Copy( tmpS, 1, mmm );  
                        end;

                        // Domain
                        Buf101 := nil; ws := nil;
                        try
                           tmpS := pNS^.Name;
                           GetMem( ws, length(tmpS)*2+1 );
                           StringToWideChar(tmpS, ws, length(tmpS)*2+1);

                           Ret101 := NetWkstaGetInfo( ws, 101, Buf101 );
                           if Ret101 = NERR_SUCCESS then begin
                              WI101 := Buf101;
                              tmpS := WideCharToString( WI101^.wki101_langroup );
                              if tmpS <> '' then begin
                                 if length( tmpS ) > sizeof(pNS^.Domain) then
                                    mmm := sizeof(pNS^.Domain)
                                 else
                                   mmm := length( tmpS );
                                 pNS^.Domain := Copy( tmpS, 1, mmm );  
                              end;

                           end;
                        finally
                           if Assigned( Buf101 ) then
                              NetApiBufferFree( Buf101 );
                           if Assigned( ws ) then
                              FreeMem( ws );
                        end;

                        // Adding to list
                        with Result.LockList do
                           try
                              Add( pNS );
                           finally
                              Result.UnlockList;
                           end;

                     end
                     else
                        Raise Exception.Create( 'Error getting memory for NetServer data' );
                  end; {if ( SI101^.sv101_type and SrvType <> 0 ) then begin}
                  inc( SI101 );
               end; {for i := 0 to EntiesRead - 1 do begin}
            end; {if ( (Ret = ERROR_MORE_DATA) or (Ret = NERR_SUCCESS) ) then begin}
         finally
            if Assigned( Buf ) then begin
               NetApiBufferFree( Buf );
               Buf := nil;
            end;
         end;
      end; {while Ret = ERROR_MORE_DATA do begin}

      if Ret <> NERR_SUCCESS then begin
         if Assigned( Result ) then
            FreeAndNilList( Result );
         Raise Exception.Create('Can''t obtain servers inforamtion');
      end;
   finally
      if Assigned( Buf ) then
         NetApiBufferFree( Buf );
      if Assigned( wc ) then
         FreeMem( wc );
      if Assigned( dom ) then
         FreeMem( dom );

   end;
end;

function TMyNetWorks.GetDomains: TThreadList;
var
   Res:          PNetResourceA;   // _NETRESOURCEA, PNetResourceA
begin
   Res := nil;
   Result := TThreadList.Create;

   EnumDomains( Result, Res );
end;

function TMyNetWorks.EnumDomains( l: TThreadList; Res: PNetResourceA ): Boolean;
var
   Hnd:          dword;
   Domains:      Pointer;
   DomainsMax:   dword;
   DomainsSize:  dword;
   Ret:          dword;
   i:            dword;
   rp:           PNetResourceA;
   NR:           PMyNetResource;
   mmm:          cardinal;
begin
   NR := nil; rp := nil; Result := False;
   Hnd := 0; DomainsMax := $FFFFFFFF; Domains := nil;
   DomainsSize := sizeof(_NETRESOURCEA)*100;
   try
      GetMem( Domains, DomainsSize );
      FillChar( Domains^, DomainsSize, 0 );
      rp := Domains;

      WNetOpenEnum( RESOURCE_GLOBALNET, RESOURCETYPE_DISK, RESOURCEUSAGE_CONTAINER, Res, Hnd );
      while 1=1 do begin
         Ret	:= WNetEnumResource( Hnd, DomainsMax, Domains, DomainsSize );
         if Ret = ERROR_NO_MORE_ITEMS then
            break
         else if Ret <> NO_ERROR then
            break
         else begin
            for i := 0 to DomainsMax-1 do begin
               if rp^.dwDisplayType = RESOURCEDISPLAYTYPE_DOMAIN then begin
                  GetMem( NR, sizeof(TMyNetResource) );
                  FillChar( NR^, sizeof(TMyNetResource), 0 );
                  NR^.Scope := rp^.dwScope;
                  NR^.Tip := rp^.dwType;
                  NR^.DisplayType := rp^.dwDisplayType;
                  NR^.Usage := rp^.dwUsage;
                  if rp^.lpLocalName <> nil then begin
                     if length(rp^.lpLocalName) > sizeof(NR^.LocalName) then mmm := sizeof(NR^.LocalName) else mmm := length(rp^.lpLocalName);
                     move( rp^.lpLocalName^, NR^.LocalName[1], mmm );
                     SetLength( NR^.LocalName, mmm );
                     //NR^.LocalName := rp^.lpLocalName;
                  end;
                  if rp^.lpRemoteName <> nil then begin
                     if length(rp^.lpRemoteName) > sizeof(NR^.RemoteName) then mmm := sizeof(NR^.RemoteName) else mmm := length(rp^.lpRemoteName);
                     move( rp^.lpRemoteName^, NR^.RemoteName[1], mmm );
                     SetLength( NR^.RemoteName, mmm );
                     //NR^.RemoteName := rp^.lpRemoteName;
                  end;
                  if rp^.lpComment <> nil then begin
                     if length(rp^.lpComment) > sizeof(NR^.Comment) then mmm := sizeof(NR^.Comment) else mmm := length(rp^.lpComment);
                     move( rp^.lpComment^, NR^.Comment[1], mmm );
                     SetLength( NR^.Comment, mmm );
                     //NR^.Comment := rp^.lpComment;
                  end;
                  if rp^.lpProvider <> nil then begin
                     if length(rp^.lpProvider) > sizeof(NR^.Provider) then mmm := sizeof(NR^.Provider) else mmm := length(rp^.lpProvider);
                     move( rp^.lpProvider^, NR^.Provider[1], mmm );
                     SetLength( NR^.Provider, mmm );
                     //NR^.Provider := rp^.lpProvider;
                  end;
                  // Adding to list
                  with l.LockList do
                     try
                        Add( NR );
                     finally
                        l.UnlockList;
                     end;
                  continue;
               end;
               if ( rp^.dwUsage and RESOURCEUSAGE_CONTAINER = RESOURCEUSAGE_CONTAINER ) then begin
                  EnumDomains( l, rp );
               end;
               inc( rp );
            end;
         end;
      end;
      WNetCloseEnum( Hnd );
      Result := True;
   finally
      if Assigned( Domains ) then
         FreeMem( Domains );
   end;
end;

function  TMyNetWorks.GetComputersEx( Domain: string; SrvType: dword=SV_TYPE_ALL ): TThreadList;
var
   Res:          PNetResourceA;   // _NETRESOURCEA, PNetResourceA
   i:            cardinal;
   SI101:        PSrvInfo101;
   Buf:          Pointer;
   Ret:          dword;
   w1:           array[0..50] of char;
   p:            PMyNetResource;
   NewRes:       TThreadList;
begin
   Result := TThreadList.Create;
   GetMem(Res, sizeof(_NETRESOURCEA));
   FillChar( Res^, sizeof(_NETRESOURCEA), 0 );
   Res^.dwScope := RESOURCE_GLOBALNET;
   Res^.dwType := RESOURCETYPE_ANY;
   Res^.dwDisplayType := RESOURCEDISPLAYTYPE_DOMAIN;
   Res^.dwUsage := RESOURCEUSAGE_CONTAINER;
   Res^.lpLocalName := nil;
   Res^.lpRemoteName := Addr( Domain[1] );

   EnumComputers( Result, Res );
   if Assigned( Result ) then begin
      NewRes := TThreadList.Create;
      with Result.LockList do begin
         while Count > 0 do begin
            p := First;
            StringToWideChar( p^.RemoteName, Addr(w1), 50 );
            try
               Ret := NetServerGetInfo( Addr(w1), 101, Buf );
               if Ret = NERR_SUCCESS then begin
                  SI101 := Buf;
                  if ( SI101^.sv101_type and SrvType <> 0 ) then begin
                     with NewRes.LockList do
                        try
                           Add( p );
                        finally
                           NewRes.UnlockList;
                        end;
                     Remove( p );   
                  end
                  else begin
                     // not need By mask (SrvType)
                     Remove( p );
                     FreeMem( p );
                  end;
               end {if Ret = NERR_SUCCESS then begin}
               else begin
                  Remove( p );
                  FreeMem( p );
               end;
            finally
               if Assigned( Buf ) then
                  NetApiBufferFree( Buf );
            end;
         end; {while Count > 0 do begin}
         Result.UnlockList;
      end; {with Result.LockList do begin}
   end;

   FreeAndNilList( Result );

   with NewRes.LockList do
      try
         if Count > 0 then
            Result := NewRes;
      finally
         NewRes.UnlockList;
      end;
end;

function  TMyNetWorks.GetComputers( Domain: string ): TThreadList;
var
   Res:          PNetResourceA;   // _NETRESOURCEA, PNetResourceA
begin
   Result := TThreadList.Create;
   GetMem(Res, sizeof(_NETRESOURCEA));
   FillChar( Res^, sizeof(_NETRESOURCEA), 0 );
   Res^.dwScope := RESOURCE_GLOBALNET;
   Res^.dwType := RESOURCETYPE_ANY;
   Res^.dwDisplayType := RESOURCEDISPLAYTYPE_DOMAIN;
   Res^.dwUsage := RESOURCEUSAGE_CONTAINER;
   Res^.lpLocalName := nil;
   Res^.lpRemoteName := Addr( Domain[1] );

   EnumComputers( Result, Res );
end;

function  TMyNetWorks.EnumComputers( l: TThreadList; Res: PNetResourceA ): Boolean;
var
   Hnd:          dword;
   Comps:        Pointer;
   CompsMax:     dword;
   CompsSize:    dword;
   Ret:          dword;
   i:            dword;
   rp:           PNetResourceA;
   NR:           PMyNetResource;
   mmm:          cardinal;
begin
   NR := nil; rp := nil; Result := False;
   Hnd := 0; CompsMax := $FFFFFFFF; Comps := nil;
   CompsSize := sizeof(_NETRESOURCEA)*512;
   try
      GetMem( Comps, CompsSize );
      FillChar( Comps^, CompsSize, 0 );
      rp := Comps;

      WNetOpenEnum( RESOURCE_GLOBALNET, RESOURCETYPE_DISK, RESOURCEUSAGE_CONTAINER, Res, Hnd );
      while 1=1 do begin
         Ret	:= WNetEnumResource( Hnd, CompsMax, Comps, CompsSize );
         if Ret = ERROR_NO_MORE_ITEMS then
            break
         else if Ret <> NO_ERROR then
            break
         else begin
            for i := 0 to CompsMax-1 do begin
               if rp^.dwDisplayType = RESOURCEDISPLAYTYPE_SERVER then begin
                  // Found Computer
                  GetMem( NR, sizeof(TMyNetResource) );
                  FillChar( NR^, sizeof(TMyNetResource), 0 );
                  NR^.Scope := rp^.dwScope;
                  NR^.Tip := rp^.dwType;
                  NR^.DisplayType := rp^.dwDisplayType;
                  NR^.Usage := rp^.dwUsage;
                  if rp^.lpLocalName <> nil then begin
                     if length(rp^.lpLocalName) > sizeof(NR^.LocalName) then mmm := sizeof(NR^.LocalName) else mmm := length(rp^.lpLocalName);
                     move( rp^.lpLocalName^, NR^.LocalName[1], mmm );
                     SetLength( NR^.LocalName, mmm );
                     //NR^.LocalName := rp^.lpLocalName;
                  end;
                  if rp^.lpRemoteName <> nil then begin
                     if length(rp^.lpRemoteName) > sizeof(NR^.RemoteName) then mmm := sizeof(NR^.RemoteName) else mmm := length(rp^.lpRemoteName);
                     move( rp^.lpRemoteName^, NR^.RemoteName[1], mmm );
                     SetLength( NR^.RemoteName, mmm );
                     //NR^.RemoteName := rp^.lpRemoteName;
                  end;
                  if rp^.lpComment <> nil then begin
                     if length(rp^.lpComment) > sizeof(NR^.Comment) then mmm := sizeof(NR^.Comment) else mmm := length(rp^.lpComment);
                     move( rp^.lpComment^, NR^.Comment[1], mmm );
                     SetLength( NR^.Comment, mmm );
                     //NR^.Comment := rp^.lpComment;
                  end;
                  if rp^.lpProvider <> nil then begin
                     if length(rp^.lpProvider) > sizeof(NR^.Provider) then mmm := sizeof(NR^.Provider) else mmm := length(rp^.lpProvider);
                     move( rp^.lpProvider^, NR^.Provider[1], mmm );
                     SetLength( NR^.Provider, mmm );
                     //NR^.Provider := rp^.lpProvider;
                  end;
                  // Adding to list
                  with l.LockList do
                     try
                        Add( NR );
                     finally
                        l.UnlockList;
                     end;
               end;
               inc( rp );
            end;
         end;
      end;
      WNetCloseEnum( Hnd );
      Result := True;
   finally
      if Assigned( Comps ) then
         FreeMem( Comps );
   end;
end;

function  TMyNetWorks.GetShares( Computer: string; WithRemotePath: Boolean = True; What: dword=RESOURCETYPE_ANY ): TThreadList;
var
   Res:          PNetResourceA;   // _NETRESOURCEA, PNetResourceA
begin
   Result := TThreadList.Create;
   GetMem(Res, sizeof(_NETRESOURCEA));
   FillChar( Res^, sizeof(_NETRESOURCEA), 0 );
   Res^.dwScope := RESOURCE_GLOBALNET;
   Res^.dwType := What;
   Res^.dwDisplayType := RESOURCEDISPLAYTYPE_DOMAIN;
   Res^.dwUsage := RESOURCEUSAGE_CONTAINER;
   Res^.lpLocalName := nil;
   Res^.lpRemoteName := Addr( Computer[1] );

   EnumShares( Result, Res, WithRemotePath, What );
end;

function  TMyNetWorks.EnumShares( l: TThreadList; Res: PNetResourceA; WithRemotePath: Boolean = True; What: dword=RESOURCETYPE_ANY ): Boolean;
var
   Hnd:          dword;
   Shares:       Pointer;
   SharesMax:    dword;
   SharesSize:   dword;
   Ret:          dword;
   i:            dword;
   rp:           PNetResourceA;
   NR:           PMyNetResource;
   Buf:          Pointer;
   SI2:          PShareInfo2;
   w1, w2:       array[0..50] of char;
   mmm:          cardinal;
begin
   NR := nil; rp := nil; Result := False;
   Hnd := 0; SharesMax := $FFFFFFFF; Shares := nil;
   SharesSize := sizeof(_NETRESOURCEA)*512;
   try
      GetMem( Shares, SharesSize );
      FillChar( Shares^, SharesSize, 0 );
      rp := Shares;

      WNetOpenEnum( RESOURCE_GLOBALNET, What, 0, Res, Hnd );
      while 1=1 do begin
         Ret	:= WNetEnumResource( Hnd, SharesMax, Shares, SharesSize );
         if Ret = ERROR_NO_MORE_ITEMS then
            break
         else if Ret <> NO_ERROR then
            break
         else begin
            for i := 0 to SharesMax-1 do begin
               if rp^.dwDisplayType = RESOURCEDISPLAYTYPE_SHARE then begin
                  // Found Computer
                  GetMem( NR, sizeof(TMyNetResource) );
                  FillChar( NR^, sizeof(TMyNetResource), 0 );
                  NR^.Scope := rp^.dwScope;
                  NR^.Tip := rp^.dwType;
                  NR^.DisplayType := rp^.dwDisplayType;
                  NR^.Usage := rp^.dwUsage;
                  if rp^.lpLocalName <> nil then begin
                     if length(rp^.lpLocalName) > sizeof(NR^.LocalName) then mmm := sizeof(NR^.LocalName) else mmm := length(rp^.lpLocalName);
                     move( rp^.lpLocalName^, NR^.LocalName[1], mmm );
                     SetLength( NR^.LocalName, mmm );
                     //NR^.LocalName := rp^.lpLocalName;
                  end;
                  if rp^.lpRemoteName <> nil then begin
                     if length(rp^.lpRemoteName) > sizeof(NR^.RemoteName) then mmm := sizeof(NR^.RemoteName) else mmm := length(rp^.lpRemoteName);
                     move( rp^.lpRemoteName^, NR^.RemoteName[1], mmm );
                     SetLength( NR^.RemoteName, mmm );
                     //NR^.RemoteName := rp^.lpRemoteName;
                  end;
                  if rp^.lpComment <> nil then begin
                     if length(rp^.lpComment) > sizeof(NR^.Comment) then mmm := sizeof(NR^.Comment) else mmm := length(rp^.lpComment);
                     move( rp^.lpComment^, NR^.Comment[1], mmm );
                     SetLength( NR^.Comment, mmm );
                     //NR^.Comment := rp^.lpComment;
                  end;
                  if rp^.lpProvider <> nil then begin
                     if length(rp^.lpProvider) > sizeof(NR^.Provider) then mmm := sizeof(NR^.Provider) else mmm := length(rp^.lpProvider);
                     move( rp^.lpProvider^, NR^.Provider[1], mmm );
                     SetLength( NR^.Provider, mmm );
                     //NR^.Provider := rp^.lpProvider;
                  end;

                  if WithRemotePath then
                  try
                     StringToWideChar( GetCompFromShareName( NR^.RemoteName ), Addr(w1), 50 );
                     StringToWideChar( GetShareFromShareName( NR^.RemoteName, False ), Addr(w2), 50 );
                     Ret := NetShareGetInfo( Addr(w1), Addr(w2), 2, Buf );
                     if Ret = NERR_SUCCESS then begin
                        SI2 := Buf;
                        if SI2^.shi2_path <> nil then
                           NR^.RemotePath := WideCharToString( SI2^.shi2_path );
                     end;
                  finally
                     if Assigned( Buf ) then
                        NetApiBufferFree( Buf );
                  end;

                  // Adding to list
                  with l.LockList do
                     try
                        Add( NR );
                     finally
                        l.UnlockList;
                     end;
               end;
               inc( rp );
            end;
         end;
      end;
      WNetCloseEnum( Hnd );
      Result := True;
   finally
      if Assigned( Shares ) then
         FreeMem( Shares );
   end;
end;


function  TMyNetWorks.ChangeUserPass( DomainOrSrv, Login, oldPass, newPass: string ): Boolean;
var
   Ret:               dword;
   a,b,c,d:           Pointer;
   w1, w2, w3, w4:    array [0..50] of char;
begin
   Result := False;

   StringToWideChar( DomainOrSrv, Addr(w1), 50 );
   StringToWideChar( Login, Addr(w2), 50 );
   StringToWideChar( oldPass, Addr(w3), 50 );
   StringToWideChar( newPass, Addr(w4), 50 );

   a := nil; b := nil;
   if DomainOrSrv <> '' then
      a := Addr( w1 );
   if Login <> '' then
      b := Addr( w2 );
   c := Addr( w3 );
   d := Addr( w4 );

   Ret := NetUserChangePassword( a, b, c, d );
   if Ret = NERR_SUCCESS then
      Result := True
end;

function TMyNetWorks.MyComputerName:String;
var
  le:DWORD;
begin
  le:=MAX_COMPUTERNAME_LENGTH+1;
  SetLength(result, le);
  if GetComputerName(PChar(result),le) then SetLength(result, le) else result:='UNKNOWN';
end;

function TMyNetWorks.MyUserName:String;
var
  le:DWORD;
begin
   le:=63;
   SetLength(result,le);
   if GetUsername(PChar(result),le) then SetLength(result, le-1) else result:='UNKNOWN';
end;

function  TMyNetWorks.GetPDC( Domain: string=''; Srv: string='' ): string;
var
   wc, wc1, wc2:  PWideChar;
   Buf:           Pointer;
   Ret:           dword;
begin
   Result := ''; wc := nil; wc1 := nil;
   if Domain <> '' then begin
      GetMem( wc, length(Domain)*2+1 );
      StringToWideChar(Domain, wc, length(Domain)*2+1);
   end;
   if Domain <> '' then begin
      GetMem( wc, length(Srv)*2+1 );
      StringToWideChar(Srv, wc, length(Srv)*2+1);
   end;

   try
      Ret := NetGetAnyDCName( nil, nil, Buf );
      if Ret = NERR_Success then begin
         wc2 := Buf;
         Result := WideCharToString( wc2 );
      end;
   finally
      if Assigned( Buf ) then
         NetApiBufferFree( Buf );
      if Assigned( wc ) then
         FreeMem( wc );
      if Assigned( wc1 ) then
         FreeMem( wc1 );
   end;
      
end;

function  TMyNetWorks.GetPDC_long_but_safely( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_DOMAIN_CTRL );
end;

function  TMyNetWorks.GetBDC( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_DOMAIN_BAKCTRL );
end;

function  TMyNetWorks.GetSQL( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_SQLSERVER );
end;

function  TMyNetWorks.GetW9x( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_WINDOWS );
end;

function  TMyNetWorks.GetWNT( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_NT );
end;

function  TMyNetWorks.GetSRV( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_SERVER );
end;

function  TMyNetWorks.GetWKS( Domain: string ): TThreadList;
begin
   Result := GetComputersEx( Domain, SV_TYPE_WORKSTATION );
end;
// ---------------------------------------------

end.
